from __future__ import unicode_literals

from .feed.factory import FeedFactory


def get_feed_items(feedconfig, limit=0):
    if feedconfig.moderated:
        qs = feedconfig.moderated_items.all()
        if limit:
            return qs[:limit]
        return qs

    stream = FeedFactory.create(feedconfig.source)
    return stream.get_items(config=feedconfig, limit=limit)



from __future__ import unicode_literals

from django import forms

from wagtail.wagtailcore import blocks

from .models import SocialFeedConfiguration
from .utils import get_feed_items


class FeedChooserBlock(blocks.ChooserBlock):
    target_model = SocialFeedConfiguration
    widget = forms.Select

    def value_for_form(self, value):
        return value.pk

    def to_python(self, value):
        return self.target_model.objects.get(pk=value)


class SocialFeedBlock(blocks.StructBlock):
    feedconfig = FeedChooserBlock()
    limit = blocks.IntegerBlock(required=False, min_value=0)

    class Meta:
        icon = 'icon icon-fa-rss'
        template = 'wagtailsocialfeed/social_feed_block.html'

    def get_context(self, value):
        context = super(SocialFeedBlock, self).get_context(value)

        feedconfig = value['feedconfig']
        context['feed'] = get_feed_items(feedconfig, limit=value['limit'])

        return context



import logging
import json

from dateutil import parser as dateparser
from django.core.exceptions import ImproperlyConfigured
from twython import Twython

from wagtailsocialfeed.utils.conf import get_socialfeed_setting

from . import AbstractFeed, FeedItem

logger = logging.getLogger('wagtailsocialfeed')


class TwitterFeedItem(FeedItem):
    @classmethod
    def from_raw(cls, raw):
        image = None
        extended = raw.get('extended_entities', None)
        if extended:
            image = process_images(extended.get('media', None))

        # Use the dateutil parser because on some platforms
        # python's own strptime doesn't support the %z directive.
        # Format: '%a %b %d %H:%M:%S %z %Y'
        date = dateparser.parse(raw.get('created_at'))
        return cls(
            id=raw['id'],
            text=raw['text'],
            image_dict=image,
            posted=date
        )


def process_images(media_dict):
    images = {}
    if not media_dict:
        return images
    base_url = media_dict[0]['media_url_https']

    # TODO: see if we can provide the width and height attributes as well
    images['small'] = dict(url=base_url + ":small")
    images['thumb'] = dict(url=base_url + ":thumb")
    images['medium'] = dict(url=base_url + ":medium")
    images['large'] = dict(url=base_url + ":large")
    return images


class TwitterFeed(AbstractFeed):
    def __init__(self, *args, **kwargs):
        super(TwitterFeed, self).__init__(*args, **kwargs)

        self.settings = get_socialfeed_setting('CONFIG').get('twitter', None)
        if not self.settings:
            raise ImproperlyConfigured(
                "No twitter configuration defined in the settings. "
                "Make sure you define WAGTAIL_SOCIALFEED_CONFIG in your "
                "settings with at least a 'twitter' entry.")

        self.twitter = Twython(self.settings['CONSUMER_KEY'],
                               self.settings['CONSUMER_SECRET'],
                               self.settings['ACCESS_TOKEN_KEY'],
                               self.settings['ACCESS_TOKEN_SECRET'])

    def fetch_online(self, config, limit=None, query_string=None, max_id=None):
        username = config.username

        def _search(tweet):
            """Very basic search function"""
            return query_string.lower() in tweet['text'].lower()

        self.iteration = 0

        def _fetch(max_id=None):
            """
            Return the raw data fetched from twitter and the oldest post in the
            result set.

            We return the oldest post as well because the raw data might be
            filtered based on the query_string, but we still need the oldest
            post to check the date to determine wether we should stop our
            search or continue
            """
            raw = self.twitter.get_user_timeline(
                screen_name=username,
                trim_user=True,
                contributor_details=False,
                include_rts=False,
                max_id=max_id)

            if not raw:
                return raw, None

            oldest_post = raw[-1]
            if query_string:
                raw = list(filter(_search, raw))
            return raw, oldest_post

        raw, oldest_post = _fetch()

        if query_string:
            # If we have a query_string, we should fetch the users timeline a
            # couple of times to dig a bit into the history.
            while oldest_post:
                # Trick from twitter API doc to exclude the oldest post from
                # the next result-set
                oldest_post_date = dateparser.parse(oldest_post.get(
                    'created_at'))
                if not self.more_history_allowed(oldest_post_date):
                    break

                max_id = oldest_post['id'] - 1
                _raw, _post = _fetch(max_id=max_id)
                if _post and _post['id'] == oldest_post['id']:
                    logger.warning("Trying to fetch older tweets but received "
                                   "same result set. Breaking the loop.")
                    break
                oldest_post = _post
                raw += _raw

        return raw

    def to_feed_item(self, raw):
        return TwitterFeedItem.from_raw(raw)



from __future__ import unicode_literals

from django.conf.urls import include, url
from django.core.urlresolvers import reverse
from django.utils import six
from django.utils.translation import ugettext_lazy as _

from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.wagtailadmin.menu import Menu, MenuItem, SubmenuMenuItem
from wagtail.wagtailcore import hooks

from . import urls
from .models import SocialFeedConfiguration
from django.db.models.signals import post_save


class SocialFeedConfigurationAdmin(ModelAdmin):
    model = SocialFeedConfiguration
    menu_label = 'Social feeds'  # TODO ditch this to use verbose_name_plural from model # NOQA
    menu_icon = 'icon icon-fa-rss'
    menu_order = 400
    add_to_settings_menu = True
    list_display = ('source', 'username')
    list_filter = ('source', )


class SocialFeedModerateMenu(Menu):
    config_menu_items = {
        # Contains all submenu items mapped to the id's of the
        # `SocialFeedConfiguration` they link to
    }

    def __init__(self):
        # Iterate over existing configurations that are moderated
        config_qs = SocialFeedConfiguration.objects.filter(moderated=True)
        for config in config_qs:
            self.config_menu_items[config.id] = \
                self._create_moderate_menu_item(config)

        self._registered_menu_items = list(self.config_menu_items.values())
        self.construct_hook_name = None

        post_save.connect(self._update_menu, sender=SocialFeedConfiguration)

    def _update_menu(self, instance, **kwargs):
        """
        Call whenever a `SocialFeedCongiration` gets changed.

        When it is not moderated anymore, but exists in our menu, it should be
        removed.
        When it is moderated but does not exist yet, we should create a
        new menu item.
        """
        has_menu_item = instance.id in self.config_menu_items.keys()

        if not instance.moderated and has_menu_item:
            menu_item = self.config_menu_items.pop(instance.id)
            index = self._registered_menu_items.index(menu_item)
            del self._registered_menu_items[index]
        elif instance.moderated and not has_menu_item:
            menu_item = self._create_moderate_menu_item(instance)
            self.config_menu_items[instance.id] = menu_item
            self._registered_menu_items.append(menu_item)

    def _create_moderate_menu_item(self, config):
        """Create a submenu item for the moderate admin page."""
        url = reverse('wagtailsocialfeed:moderate', kwargs={'pk': config.pk})
        return MenuItem(six.text_type(config), url,
                        classnames='icon icon-folder-inverse')


modeladmin_register(SocialFeedConfigurationAdmin)


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^socialfeed/', include(urls, app_name='wagtailsocialfeed',
            namespace='wagtailsocialfeed')),
    ]


@hooks.register('register_admin_menu_item')
def register_socialfeed_menu():
    # Create the main socialfeed menu item
    socialfeed_menu = SocialFeedModerateMenu()
    return SubmenuMenuItem(
        _('Social feed'), socialfeed_menu, classnames='icon icon-fa-rss',
        order=800)



from .twitter import TwitterFeed, TwitterFeedItem
from .instagram import InstagramFeed, InstagramFeedItem

from collections import namedtuple

FeedClasses = namedtuple('FeedClasses', ['feed', 'item'])

FEED_CONFIG = {
    'twitter': FeedClasses(TwitterFeed, TwitterFeedItem),
    'instagram': FeedClasses(InstagramFeed, InstagramFeedItem)
}


class FeedFactory(object):
    @classmethod
    def create(cls, source):
        try:
            return FEED_CONFIG[source].feed()
        except KeyError:
            raise NotImplementedError(
                "Feed class for type '{}' not available".format(source))


class FeedItemFactory(object):
    @classmethod
    def get_class(cls, source):
        try:
            return FEED_CONFIG[source].item
        except KeyError:
            raise NotImplementedError(
                "FeedItem class for type '{}' not available".format(source))



from __future__ import unicode_literals

import json
import logging
import datetime
from dateutil.tz import tzutc

from django.core.cache import cache
from django.utils import six

from wagtailsocialfeed.utils.conf import get_socialfeed_setting

logger = logging.getLogger('wagtailsocialfeed')


def date_handler(obj):
    if hasattr(obj, 'isoformat'):
        return obj.isoformat()
    else:  # pragma: no cover
        raise TypeError


class FeedError(Exception):
    pass


class FeedItem(object):
    def __init__(self, id, text, posted, image_dict, *args, **kwargs):
        self.id = six.text_type(id)  # Ensure it's a string
        self.text = text
        self.posted = posted
        self.image_dict = image_dict

    @property
    def image(self):
        """
        Convenience property to be used in templates.

        Can be used in templates as such::

            item.image.small.url
        """
        return self.image_dict

    def serialize(self):
        return json.dumps(vars(self), default=date_handler)

    @classmethod
    def from_moderated(cls, moderated):
        """Create an `FeedItem` object from a `ModeratedItem`"""
        source = json.loads(moderated.content)

        # We could convert source['posted'] to a proper DateTime
        # object but why bother when it is saved in
        # moderated.posted as well
        source['posted'] = moderated.posted
        return cls(**source)


class AbstractFeed(object):
    """
    All feed implementations should subclass this class.

    This base class provides caching functionality.
    The subclass needs to take care of actually fetching the feed from the
    online source and converting them to `FeedItem`s.
    """

    def get_items(self, config, limit=0, query_string=None, use_cache=True):
        """
        Return a list of `FeedItem`s and handle caching.

        :param config: `SocialFeedConfiguration` to use
        :param limit: limit the output. Use 0 or None for no limit (default=0)
        :param query_string: the search term to filter on (default=None)
        :param use_cache: utilize the cache store/retreive the results
            (default=True)
        """
        cls_name = self.__class__.__name__
        cache_key = 'socialfeed:{}:data:{}'.format(cls_name, config.id)

        if query_string:
            cache_key += ":q-{}".format(query_string)

        data = cache.get(cache_key, []) if use_cache else None
        if data:
            logger.debug("Getting data from cache ({})".format(cache_key))
        else:
            logger.debug("Fetching data online")

            data_raw = self.fetch_online(config=config, limit=limit,
                                         query_string=query_string)
            data = list(map(self._convert_raw_item, data_raw))

            if use_cache:
                logger.debug("Storing data in cache ({})".format(cache_key))
                cache.set(cache_key, data,
                          get_socialfeed_setting('CACHE_DURATION'))

        if limit:
            return data[:limit]
        return data

    def more_history_allowed(self, oldest_date):
        """
        Determine if we should load more history.

        This is used when searching for specific posts
        using the `query_string` argument.

        :param oldest_date: date of the oldest post in the last set returned
        """
        now = datetime.datetime.now(tzutc())
        last_allowed = now - get_socialfeed_setting('SEARCH_MAX_HISTORY')
        return oldest_date > last_allowed

    def to_feed_item(self, raw):
        raise NotImplementedError(
            "to_feed_item() is not implemented yet by {}".format(
                self.__class__.__name__))

    def fetch_online(self, config, limit=None, query_string=None):
        raise NotImplementedError(
            "fetch_online() is not implemented yet by {}".format(
                self.__class__.__name__))

    def _convert_raw_item(self, raw):
        item = self.to_feed_item(raw)
        assert isinstance(item, FeedItem)
        return item



import datetime
import logging

import requests
from django.utils import timezone

from . import AbstractFeed, FeedError, FeedItem

logger = logging.getLogger('wagtailsocialfeed')


def _raw_timestamp_to_date(ts):
    timestamp = None
    try:
        timestamp = float(ts)
    except ValueError:
        return None

    return timezone.make_aware(
        datetime.datetime.fromtimestamp(timestamp), timezone=timezone.utc)


class InstagramFeedItem(FeedItem):
    """Implements instagram-specific behaviour"""

    @classmethod
    def from_raw(cls, raw):
        date = None
        image = {}

        if 'created_time' in raw:
            date = _raw_timestamp_to_date(raw['created_time'])

        if 'images' in raw:
            image = {
                'thumb': raw['images']['thumbnail'],
                'small': raw['images']['low_resolution'],
                'medium': raw['images']['standard_resolution'],
                'largel': None,
            }

        return cls(
            id=raw['id'],
            text=raw['caption']['text'],
            image_dict=image,
            posted=date,
        )


class InstagramFeed(AbstractFeed):
    def fetch_online(self, config, limit=None, query_string=None):
        username = config.username

        def _search(tweet):
            """Very basic search function"""
            return query_string.lower() in tweet['caption']['text'].lower()

        def _fetch(max_id=None):
            url = "https://www.instagram.com/{}/media/".format(username)
            if max_id:
                url += "?max_id={}".format(max_id)

            resp = requests.get(url)
            if resp.status_code == 200:
                try:
                    raw = resp.json()['items']
                except ValueError as e:
                    raise FeedError(e)
                except KeyError as e:
                    raise FeedError("No items could be found in the response")

                if not raw:
                    return raw, None

                oldest_post = raw[-1]
                if query_string:
                    raw = list(filter(_search, raw))
                return raw, oldest_post
            raise FeedError(resp.reason)

        raw, oldest_post = _fetch()

        if query_string:
            # If we have a query_string, we should fetch the users timeline a
            # couple of times to dig a bit into the history.
            while oldest_post:
                # Trick from twitter API doc to exclude the oldest post from
                # the next result-set
                oldest_post_date = _raw_timestamp_to_date(
                    oldest_post['created_time'])
                if not self.more_history_allowed(oldest_post_date):
                    break

                _raw, _post = _fetch(max_id=oldest_post['id'])
                if _post and _post['id'] == oldest_post['id']:
                    logger.warning("Trying to fetch older tweets but received "
                                   "same result set. Breaking the loop.")
                    break
                oldest_post = _post
                raw += _raw

        return raw

    def to_feed_item(self, raw):
        return InstagramFeedItem.from_raw(raw)
